#!/bin/sh
#set -x
                                                                                
backup () {
  local file
  file="$1"
  if [ -f "$file" ] ;then
    echo "Backup $file ..."
    if [ ! -f "$file.shipped" ] ;then
      mv -f "$file" "$file.shipped"
    fi
    cp -f "$file.shipped" "$file"
  fi
  if [ -f "$file.shipped" ] ;then
    cp -f "$file.shipped" "$file"
  fi
}

FLOPPY=`dirname $0`
FLOPPY=`( cd $FLOPPY; pwd )`

DISK="--none--"
#####################################
# find where we are and where we go #
#####################################
if [ ! -d /media/floppy ] ;then
  if [ ! -d /mnt/sysimage ] ;then
    mkdir -p /mnt/sysimage
  fi
  if [ ! -d /boot/grub -a ! -f /etc/lilo.conf ] ;then
    mount /dev/sda2 /mnt/sysimage
    if [ $? == 0 ] ;then
      if [ -d /mnt/sysimage/boot ] ;then
        if [ ! -d /mnt/sysimage/boot/grub ] ;then
          mount /dev/sda1 /mnt/sysimage/boot
        fi
      fi
    fi
  fi
fi
if [ -d /mnt/sysimage/etc ] ;then
  DISK="/mnt/sysimage"
elif [ -d /etc/X11 ] ;then
  DISK=""
fi
if [ "$DISK" == "--none--" ] ;then
  echo "Unknown system layout"
  exit 9
fi
#echo -n ",,, - "; read a

#if [ -d /var/adm/mount/linux/suse ] ;then
  ############################################
  # we have been copied to somewhere in /tmp #
  # by YaST control center's Load Vendor CD  #
  ############################################
  #FLOPPY="/var/adm/mount"
#else
  #FLOPPY=`dirname $0`
#fi

##########################
# select the driver name #
##########################
a=`cat /proc/pci | grep ":3124 "`
if [ "$a" == "" ] ;then
  sii="sii6514"
else
  sii="sii6524"
fi

############################
# copy driver to boot disk #
#   update kernel files    #
############################

#handle new RH format
if [ -f "$FLOPPY/rhdd" ] ;then
#if [ -d "$DISK/lib/modules/2.4.21-15.EL" ] ;then
  Arch="x86_64"
else
Arch=""
fi

if [ -d "/lib/modules/2.4.21-99-smp4G" ] ;then
  src="i386-9.0SMP4G"
  ver="2.4.21-99-smp4G"
elif [ -d "/lib/modules/2.4.21-99-smp" ] ;then
  src="i386-9.0SMP"
  ver="2.4.21-99-smp"
elif [ -d "/lib/modules/2.4.21-99-athlon" ] ;then
  src="i386-9.0athlon"
  ver="2.4.21-99-athlon"
elif [ -d "/lib/modules/2.4.21-102-default" ] ;then
  src="x86_64-9.0"
  ver="2.4.21-102-default"
elif [ -d "/lib/modules/2.4.21-102-smp" ] ;then
  src="x86_64-9.0SMP"
  ver="2.4.21-102-smp"
elif [ -d "/lib/modules/2.4.19-64GB-SMP" ] ;then
  src="i386-ul164GB-SMP"
  ver="2.4.19-64GB-SMP"
elif [ -f "$FLOPPY/modules.cgz" ] ;then
  #DISK="/mnt/sysimage"
  img=".img"
  mkdir /tmp/cpio
  (
    cd /tmp/cpio
    echo "Unzip drivers ..."
    cat "$FLOPPY/modules.cgz" | gunzip | cpio -ivH crc
    for dir in * ;do
      if [ -d $DISK/lib/modules/$dir ] ;then
        echo "copying $dir/$Arch/$sii.o to $DISK/lib/modules/$dir/kernel/drivers/scsi"
        cp -f $dir/$Arch/$sii.o $DISK/lib/modules/$dir/kernel/drivers/scsi
      fi
    done
  )
  rm -rf /tmp/cpio
else
  vers="2.4.21-99-smp4G 2.4.21-99-athlon 2.4.21-102-smp 2.4.13-64GB-SMP"
  echo "ERROR: we only do $vers"
  exit 9
fi
#echo -n ",,, - "; read a

dirs=`( cd $DISK/lib/modules; echo * )`
for dir in $dirs ;do
  modules_dep="$DISK/lib/modules/$dir/modules.dep"
  if [ -f "$modules_dep" ] ;then
    echo "Updating $modules_dep ..."

    ##########################################
    ## backup the origional modules.dep file #
    ##########################################
    #backup $modules_dep
    ##########################################################
    # problem with having unexpected files in /lib/modules/* #
    ##########################################################

    #####################################################
    # add the Sii driver to the modules dependency file #
    #####################################################
    a=`grep $sii $modules_dep`
    if [ "$a" == "" ] ;then
      echo "Update $modules_dep ..."
      sii_dir="/lib/modules/$dir/kernel/drivers/scsi"
      echo "$sii_dir/$sii.o: $sii_dir/scsi_mod.o" >>$modules_dep
    fi
  fi
  #echo -n ",,, - "; read a
done

############################
# copy driver to boot disk #
############################
if [ "$src" != "" ] ;then
  suse_obj="$FLOPPY/linux/*/$src/modules/$sii.o"
  echo "cp -f $suse_obj $DISK/lib/modules/$ver/kernel/drivers/scsi"
  cp -f $suse_obj $DISK/lib/modules/$ver/kernel/drivers/scsi
  #echo -n ",,, - "; read a

  #####################################
  # backup and modify the kernel file #
  #####################################
  backup $DISK/etc/sysconfig/kernel
  a=`grep $sii $DISK/etc/sysconfig/kernel`
  if [ "$a" == "" ] ;then
    echo "Update $DISK/etc/sysconfig/kernel ..."
    sed -f $FLOPPY/addsii.sed $DISK/etc/sysconfig/kernel > /tmp/kertmp
    mv -f /tmp/kertmp $DISK/etc/sysconfig/kernel
  fi
  #echo -n ",,, - "; read a
fi
#echo -n ",,, - "; read a

#########################################
# backup and modify the bootloader file #
#########################################
# this will try to force LILO #
# which is not needed anymore #
###############################
#backup /etc/sysconfig/bootloader
#sed -f $FLOPPY/add2sii.sed /etc/sysconfig/bootloader > /tmp/kertmp2
#mv -f /tmp/kertmp2 /etc/sysconfig/bootloader

#######################################
# backup and generate the initrd file #
#######################################
dirs=`( cd $DISK/lib/modules; echo * )`
for dir in $dirs ;do
  initrd="initrd-$dir$img"
  if [ -d $DISK/lib/modules/$dir/kernel ] ;then
    sysmap="System.map-$dir"
    backup $DISK/boot/$initrd
    backup $DISK/boot/$sysmap
    #echo -n ",,, - "; read a
    echo "mkinitrd $initrd ..."
    if [ "$src" == "" ] ;then
      ##################
      # Redhat version #
      ##################
      rm -f $DISK/boot/$initrd
      if [ "$DISK" == "" ] ;then
        disk="/"
      else
        disk="$DISK"
      fi
      $DISK/usr/sbin/chroot $disk /sbin/mkinitrd --with=scsi_mod --with=$sii /boot/$initrd $dir
    else
      ################
      # SuSE version #
      ################
      ( cd /boot; /sbin/mkinitrd )
    fi
  fi
  #echo -n ",,, - "; read a
done

##########################
# update the boot loader #
##########################
echo "The next section updates the bootloader"
echo "This section is not normally required"
echo "Modify the script if you need it"
echo "DONE"

#cp -f $FLOPPY/special_update_bootloader.sh /tmp
#chmod 775 /tmp/special_update_bootloader.sh
#$FLOPPY/special_update_bootloader.sh
